# Copyright 2022 neomadas-dev
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#   3. Neither the name of the copyright holder nor the names of its
#      contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Operator specification tests."""

from __future__ import annotations

from unittest import TestCase

from .spec_common.false_spec import FalseSpec
from .spec_common.true_spec import TrueSpec


class OperatorSpecificationTestCase(TestCase):
    def setUp(self):
        """Start basic specifications."""
        self.trueSpec = TrueSpec()
        self.falseSpec = FalseSpec()

    def test_and(self):
        """Test and operator."""
        self.assertTrue(
            self.trueSpec.And(self.trueSpec).IsSatisfiedBy(object())
        )
        self.assertFalse(
            self.trueSpec.And(self.falseSpec).IsSatisfiedBy(object())
        )
        self.assertFalse(
            self.falseSpec.And(self.trueSpec).IsSatisfiedBy(object())
        )
        self.assertFalse(
            self.falseSpec.And(self.falseSpec).IsSatisfiedBy(object())
        )

    def test_or(self):
        """Test or operator."""
        self.assertTrue(self.trueSpec.Or(self.trueSpec).IsSatisfiedBy(object()))
        self.assertTrue(
            self.trueSpec.Or(self.falseSpec).IsSatisfiedBy(object())
        )
        self.assertTrue(
            self.falseSpec.Or(self.trueSpec).IsSatisfiedBy(object())
        )
        self.assertFalse(
            self.falseSpec.Or(self.falseSpec).IsSatisfiedBy(object())
        )

    def test_not(self):
        """Test not operator."""
        self.assertFalse(self.trueSpec.Not().IsSatisfiedBy(object()))
        self.assertTrue(self.falseSpec.Not().IsSatisfiedBy(object()))
